@@ -24,6 +24,7 @@
 #endif
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <errno.h>
 #include <string.h>
 #include <net/if.h>
@@ -35,20 +36,28 @@
 #include <connman/inet.h>
 #include <connman/log.h>
 
+#include <WiMaxType.h>
 #include <WiMaxAPI.h>
 #include <WiMaxAPIEx.h>
 
 #include "iwmx.h"
 
 /* Yes, this is dirty; see above on IWMX_SDK_DEV_MAX*/
-static struct wmxsdk g_iwmx_sdk_devs[IWMX_SDK_DEV_MAX];
+static struct wmxsdk *g_iwmx_sdk_devs[IWMX_SDK_DEV_MAX];
 
-static struct wmxsdk *deviceid_to_wmxsdk(struct WIMAX_API_DEVICE_ID *device_id)
+static struct wmxsdk *deviceid_to_wmxsdk(WIMAX_API_DEVICE_ID *device_id)
 {
-	return container_of(device_id, struct wmxsdk, device_id);
+	unsigned cnt;
+	for (cnt = 0; cnt < IWMX_SDK_DEV_MAX; cnt++) {
+		struct wmxsdk *wmxsdk = g_iwmx_sdk_devs[cnt];
+		if (wmxsdk &&
+		    wmxsdk->device_id.deviceIndex == device_id->deviceIndex)
+			return wmxsdk;
+	}
+	return NULL;
 }
 
-static struct WIMAX_API_DEVICE_ID g_api;
+static WIMAX_API_DEVICE_ID g_api;
 
 
 /*
@@ -98,8 +107,10 @@ const char *iwmx_sdk_dev_status_to_str(WIMAX_API_DEVICE_STATUS status)
 		return "Connection in progress";
 	case WIMAX_API_DEVICE_STATUS_Data_Connected:
 		return "Layer 2 connected";
+#if HAVE_IWMXSDK_STATUS_IDLE
 	case WIMAX_API_DEVICE_STATUS_Connection_Idle:
 		return "Idle connection";
+#endif /* #if HAVE_IWMXSDK_STATUS_IDLE */
 	default:
 		return "unknown state";
 	}
@@ -183,7 +194,7 @@ struct connman_network *__iwmx_sdk_get_connected_network(struct wmxsdk *wmxsdk)
 {
 	struct connman_network *nw;
 
-	struct WIMAX_API_CONNECTED_NSP_INFO nsp_info;
+	WIMAX_API_CONNECTED_NSP_INFO nsp_info;
 	WIMAX_API_RET r;
 	char errstr[512];
 	UINT32 errstr_size = sizeof(errstr);
@@ -224,8 +235,8 @@ struct connman_network *__iwmx_sdk_get_connected_network(struct wmxsdk *wmxsdk)
  * change callback is called and that will fiddle with the connman
  * internals.
  */
-static void __iwmx_sdk_rf_state_cb(struct WIMAX_API_DEVICE_ID *device_id,
-						WIMAX_API_RF_STATE rf_state)
+static void __iwmx_sdk_rf_state_cb(WIMAX_API_DEVICE_ID *device_id,
+				   WIMAX_API_RF_STATE rf_state)
 {
 	DBG("rf_state changed to %d\n", rf_state);
 }
@@ -298,7 +309,9 @@ int iwmx_sdk_rf_state_set(struct wmxsdk *wmxsdk, WIMAX_API_RF_STATE rf_state)
 	case WIMAX_API_DEVICE_STATUS_Scanning:
 	case WIMAX_API_DEVICE_STATUS_Connecting:
 	case WIMAX_API_DEVICE_STATUS_Data_Connected:
+#if HAVE_IWMXSDK_STATUS_IDLE
 	case WIMAX_API_DEVICE_STATUS_Connection_Idle:
+#endif
 		if (rf_state == WIMAX_API_RF_ON) {
 			result = 0;
 			DBG("radio is already on\n");
@@ -337,8 +350,8 @@ error_get_status:
  * from disconnecting to something else); the state change callback is
  * called and that will fiddle with the connman internals.
  */
-static void __iwmx_sdk_connect_cb(struct WIMAX_API_DEVICE_ID *device_id,
-					WIMAX_API_NETWORK_CONNECTION_RESP resp)
+static void __iwmx_sdk_connect_cb(WIMAX_API_DEVICE_ID *device_id,
+				  WIMAX_API_NETWORK_CONNECTION_RESP resp)
 {
 	WIMAX_API_DEVICE_STATUS status;
 	struct wmxsdk *wmxsdk = deviceid_to_wmxsdk(device_id);
@@ -346,7 +359,10 @@ static void __iwmx_sdk_connect_cb(struct WIMAX_API_DEVICE_ID *device_id,
 	status = iwmx_cm_status_get(wmxsdk);
 	if (resp == WIMAX_API_CONNECTION_SUCCESS) {
 		if (status != WIMAX_API_DEVICE_STATUS_Data_Connected
-		    && status != WIMAX_API_DEVICE_STATUS_Connection_Idle)
+#if HAVE_IWMXSDK_STATUS_IDLE
+		    && status != WIMAX_API_DEVICE_STATUS_Connection_Idle
+#endif
+			)
 			connman_error("wmxsdk: error: connect worked, but state"
 				      " didn't change (now it is %d [%s])\n",
 				      status,
@@ -406,7 +422,9 @@ int iwmx_sdk_connect(struct wmxsdk *wmxsdk, struct connman_network *nw)
 		result = -EINPROGRESS;
 		goto error_cant_do;
 	case WIMAX_API_DEVICE_STATUS_Data_Connected:
+#if HAVE_IWMXSDK_STATUS_IDLE
 	case WIMAX_API_DEVICE_STATUS_Connection_Idle:
+#endif
 		connman_error("wmxsdk: BUG? need to disconnect?\n");
 		result = -EINVAL;
 		goto error_cant_do;
@@ -450,8 +468,8 @@ error_get_status:
  * We just update the result of the command and wake up anybody who is
  * waiting for this conditional variable.
  */
-static void __iwmx_sdk_disconnect_cb(struct WIMAX_API_DEVICE_ID *device_id,
-					WIMAX_API_NETWORK_CONNECTION_RESP resp)
+static void __iwmx_sdk_disconnect_cb(WIMAX_API_DEVICE_ID *device_id,
+				     WIMAX_API_NETWORK_CONNECTION_RESP resp)
 {
 	struct wmxsdk *wmxsdk = deviceid_to_wmxsdk(device_id);
 	WIMAX_API_DEVICE_STATUS status;
@@ -459,7 +477,10 @@ static void __iwmx_sdk_disconnect_cb(struct WIMAX_API_DEVICE_ID *device_id,
 	status = iwmx_cm_status_get(wmxsdk);
 	if (resp == WIMAX_API_CONNECTION_SUCCESS) {
 		if (status == WIMAX_API_DEVICE_STATUS_Data_Connected
-		    || status == WIMAX_API_DEVICE_STATUS_Connection_Idle)
+#if HAVE_IWMXSDK_STATUS_IDLE
+		    || status == WIMAX_API_DEVICE_STATUS_Connection_Idle
+#endif
+			)
 			connman_error("wmxsdk: error: disconnect worked, "
 				      "but state didn't change (now it is "
 				      "%d [%s])\n", status,
@@ -510,7 +531,9 @@ int iwmx_sdk_disconnect(struct wmxsdk *wmxsdk)
 		goto error_cant_do;
 	case WIMAX_API_DEVICE_STATUS_Connecting:
 	case WIMAX_API_DEVICE_STATUS_Data_Connected:
+#if HAVE_IWMXSDK_STATUS_IDLE
 	case WIMAX_API_DEVICE_STATUS_Connection_Idle:
+#endif
 		break;
 	default:
 		g_assert(1);
@@ -535,7 +558,7 @@ error_get_status:
  *
  * Just pass them to the state transition handler
  */
-static void __iwmx_sdk_state_change_cb(struct WIMAX_API_DEVICE_ID *device_id,
+static void __iwmx_sdk_state_change_cb(WIMAX_API_DEVICE_ID *device_id,
 					WIMAX_API_DEVICE_STATUS status,
 					WIMAX_API_STATUS_REASON reason,
 					WIMAX_API_CONNECTION_PROGRESS_INFO pi)
@@ -551,9 +574,9 @@ static void __iwmx_sdk_state_change_cb(struct WIMAX_API_DEVICE_ID *device_id,
  * From here we update the connman core idea of which networks are
  * available.
  */
-static void __iwmx_sdk_scan_common_cb(struct WIMAX_API_DEVICE_ID *device_id,
-					struct WIMAX_API_NSP_INFO_EX *nsp_list,
-							UINT32 nsp_list_size)
+static void __iwmx_sdk_scan_common_cb(WIMAX_API_DEVICE_ID *device_id,
+				      WIMAX_API_NSP_INFO_EX *nsp_list,
+				      UINT32 nsp_list_size)
 {
 	struct wmxsdk *wmxsdk = deviceid_to_wmxsdk(device_id);
 	unsigned itr;
@@ -562,7 +585,7 @@ static void __iwmx_sdk_scan_common_cb(struct WIMAX_API_DEVICE_ID *device_id,
 	g_static_mutex_lock(&wmxsdk->network_mutex);
 	for (itr = 0; itr < nsp_list_size; itr++) {
 		int strength;
-		struct WIMAX_API_NSP_INFO_EX *nsp_info = &nsp_list[itr];
+		WIMAX_API_NSP_INFO_EX *nsp_info = &nsp_list[itr];
 		snprintf(station_name, sizeof(station_name),
 			 "%s", (char *)nsp_info->NSPName);
 		/* CAPI is reporing link quality as zero -- if it is
@@ -591,9 +614,9 @@ static void __iwmx_sdk_scan_common_cb(struct WIMAX_API_DEVICE_ID *device_id,
  *
  * We treat them same as wide, so we just call that.
  */
-static void __iwmx_sdk_wide_scan_cb(struct WIMAX_API_DEVICE_ID *device_id,
-				struct WIMAX_API_NSP_INFO_EX *nsp_list,
-							UINT32 nsp_list_size)
+static void __iwmx_sdk_wide_scan_cb(WIMAX_API_DEVICE_ID *device_id,
+				    WIMAX_API_NSP_INFO_EX *nsp_list,
+				    UINT32 nsp_list_size)
 {
 	__iwmx_sdk_scan_common_cb(device_id, nsp_list, nsp_list_size);
 }
@@ -603,8 +626,8 @@ static void __iwmx_sdk_wide_scan_cb(struct WIMAX_API_DEVICE_ID *device_id,
  *
  * We treat them same as wide, so we just call that.
  */
-static void __iwmx_sdk_scan_cb(struct WIMAX_API_DEVICE_ID *device_id,
-				struct WIMAX_API_NSP_INFO_EX *nsp_list,
+static void __iwmx_sdk_scan_cb(WIMAX_API_DEVICE_ID *device_id,
+				WIMAX_API_NSP_INFO_EX *nsp_list,
 				UINT32 nsp_list_size, UINT32 searchProgress)
 {
 	__iwmx_sdk_scan_common_cb(device_id, nsp_list, nsp_list_size);
@@ -621,7 +644,7 @@ int iwmx_sdk_scan(struct wmxsdk *wmxsdk)
 	int result;
 
 	UINT32 nsp_list_length = 10;
-	struct WIMAX_API_NSP_INFO_EX nsp_list[10];	/* FIXME: up to 32? */
+	WIMAX_API_NSP_INFO_EX nsp_list[10];	/* FIXME: up to 32? */
 
 	WIMAX_API_RET r;
 	char errstr[512];
@@ -799,13 +822,19 @@ static void iwmx_sdk_dev_add(unsigned idx, unsigned api_idx, const char *name)
 			      idx, IWMX_SDK_DEV_MAX);
 		goto error_bug;
 	}
-	wmxsdk = &g_iwmx_sdk_devs[idx];
-	if (wmxsdk->dev != NULL) {
+	if (g_iwmx_sdk_devs[idx] != NULL) {
 		connman_error("BUG! device index %u already enumerated?\n",
 			      idx);
 		goto error_bug;
 	}
 
+	wmxsdk = malloc(sizeof(*wmxsdk));
+	if (wmxsdk == NULL) {
+		connman_error("Can't allocate %zu bytes\n",
+			      sizeof(*wmxsdk));
+		goto error_bug;
+	}
+
 	memset(wmxsdk, 0, sizeof(*wmxsdk));
 	wmxsdk_init(wmxsdk);
 	/*
@@ -848,6 +877,7 @@ static void iwmx_sdk_dev_add(unsigned idx, unsigned api_idx, const char *name)
 			      wmxsdk->ifname, result);
 		goto error_dev_add;
 	}
+	g_iwmx_sdk_devs[idx] = wmxsdk;
 	return;
 
 error_dev_add:
@@ -869,7 +899,7 @@ static void iwmx_sdk_dev_rm(unsigned idx)
 			      idx, IWMX_SDK_DEV_MAX);
 		goto error_bug;
 	}
-	wmxsdk = &g_iwmx_sdk_devs[idx];
+	wmxsdk = g_iwmx_sdk_devs[idx];
 	if (wmxsdk->dev == NULL) {
 		DBG("device index %u not enumerated? ignoring\n", idx);
 		goto error_bug;
@@ -879,16 +909,18 @@ static void iwmx_sdk_dev_rm(unsigned idx)
 	wmxsdk->name[0] = 0;
 	connman_device_unref(wmxsdk->dev);
 	memset(wmxsdk, 0, sizeof(*wmxsdk));
+	g_iwmx_sdk_devs[idx] = NULL;
+	free(wmxsdk);
 error_bug:
 	return;
 }
 
-static void iwmx_sdk_addremove_cb(struct WIMAX_API_DEVICE_ID *devid,
-								BOOL presence)
+static void iwmx_sdk_addremove_cb(WIMAX_API_DEVICE_ID *devid,
+				  BOOL presence)
 {
 	unsigned int cnt;
 	WIMAX_API_RET r;
-	struct WIMAX_API_HW_DEVICE_ID device_id_list[5];
+	WIMAX_API_HW_DEVICE_ID device_id_list[5];
 	UINT32 device_id_list_size = ARRAY_SIZE(device_id_list);
 
 	char errstr[512];
@@ -909,7 +941,7 @@ static void iwmx_sdk_addremove_cb(struct WIMAX_API_DEVICE_ID *devid,
 		DBG("No WiMAX devices reported\n");
 	else
 		for (cnt = 0; cnt < device_id_list_size; cnt++) {
-			struct WIMAX_API_HW_DEVICE_ID *dev =
+			WIMAX_API_HW_DEVICE_ID *dev =
 				device_id_list + cnt;
 			DBG("#%u index #%u device %s\n",
 			    cnt, dev->deviceIndex, dev->deviceName);
@@ -922,7 +954,7 @@ static void iwmx_sdk_addremove_cb(struct WIMAX_API_DEVICE_ID *devid,
 	}
 
 	if (presence) {
-		struct WIMAX_API_HW_DEVICE_ID *dev =
+		WIMAX_API_HW_DEVICE_ID *dev =
 			device_id_list + devid->deviceIndex;
 		iwmx_sdk_dev_add(devid->deviceIndex, dev->deviceIndex,
 			       dev->deviceName);
@@ -943,7 +975,7 @@ int iwmx_sdk_api_init(void)
 	char errstr[512];
 	UINT32 errstr_size = sizeof(errstr);
 
-	struct WIMAX_API_HW_DEVICE_ID device_id_list[5];
+	WIMAX_API_HW_DEVICE_ID device_id_list[5];
 	UINT32 device_id_list_size = ARRAY_SIZE(device_id_list);
 
 	memset(&g_api, 0, sizeof(g_api));
@@ -983,7 +1015,7 @@ int iwmx_sdk_api_init(void)
 		DBG("No WiMAX devices reported\n");
 	else
 		for (cnt = 0; cnt < device_id_list_size; cnt++) {
-			struct WIMAX_API_HW_DEVICE_ID *dev =
+			WIMAX_API_HW_DEVICE_ID *dev =
 				device_id_list + cnt;
 			DBG("#%u index #%u device %s\n",
 			    cnt, dev->deviceIndex, dev->deviceName);
