@@ -39,6 +39,9 @@
 #include <connman/plugin.h>
 #include <connman/log.h>
 
+static in_addr_t loopback_address;
+static in_addr_t loopback_netmask;
+
 #if 0
 static GIOChannel *inotify_channel = NULL;
 
@@ -173,10 +176,40 @@ static int setup_hostname(void)
 	return 0;
 }
 
+static gboolean valid_loopback(int sk, struct ifreq *ifr)
+{
+	struct sockaddr_in *addr;
+	int err;
+
+	err = ioctl(sk, SIOCGIFADDR, ifr);
+	if (err < 0) {
+		err = -errno;
+		connman_error("Getting address failed (%s)", strerror(-err));
+		return TRUE;
+	}
+
+	addr = (struct sockaddr_in *) &ifr->ifr_addr;
+	if (addr->sin_addr.s_addr != loopback_address)
+		return FALSE;
+
+	err = ioctl(sk, SIOCGIFNETMASK, ifr);
+	if (err < 0) {
+		err = -errno;
+		connman_error("Getting netmask failed (%s)", strerror(-err));
+		return TRUE;
+	}
+
+	addr = (struct sockaddr_in *) &ifr->ifr_netmask;
+	if (addr->sin_addr.s_addr != loopback_netmask)
+		return FALSE;
+
+	return TRUE;
+}
+
 static int setup_loopback(void)
 {
 	struct ifreq ifr;
-	struct sockaddr_in *addr;
+	struct sockaddr_in addr;
 	int sk, err;
 
 	sk = socket(PF_INET, SOCK_DGRAM, 0);
@@ -192,14 +225,19 @@ static int setup_loopback(void)
 	}
 
 	if (ifr.ifr_flags & IFF_UP) {
-		err = -EALREADY;
-		connman_info("The loopback interface is already up");
-		goto done;
+		connman_info("Checking loopback interface settings");
+		if (valid_loopback(sk, &ifr) == TRUE) {
+			err = -EALREADY;
+			goto done;
+		}
+
+		connman_warn("Correcting wrong lookback settings");
 	}
 
-	addr = (struct sockaddr_in *) &ifr.ifr_addr;
-	addr->sin_family = AF_INET;
-	addr->sin_addr.s_addr = inet_addr("127.0.0.1");
+	memset(&addr, 0, sizeof(addr));
+	addr.sin_family = AF_INET;
+	addr.sin_addr.s_addr = inet_addr("127.0.0.1");
+	memcpy(&ifr.ifr_addr, &addr, sizeof(ifr.ifr_addr));
 
 	err = ioctl(sk, SIOCSIFADDR, &ifr);
 	if (err < 0) {
@@ -208,9 +246,10 @@ static int setup_loopback(void)
 		goto done;
 	}
 
-	addr = (struct sockaddr_in *) &ifr.ifr_netmask;
-	addr->sin_family = AF_INET;
-	addr->sin_addr.s_addr = inet_addr("255.0.0.0");
+	memset(&addr, 0, sizeof(addr));
+	addr.sin_family = AF_INET;
+	addr.sin_addr.s_addr = inet_addr("255.0.0.0");
+	memcpy(&ifr.ifr_netmask, &addr, sizeof(ifr.ifr_netmask));
 
 	err = ioctl(sk, SIOCSIFNETMASK, &ifr);
 	if (err < 0) {
@@ -241,6 +280,9 @@ done:
 
 static int loopback_init(void)
 {
+	loopback_address = inet_addr("127.0.0.1");
+	loopback_netmask = inet_addr("255.0.0.0");
+
 	setup_loopback();
 
 	setup_hostname();
