@@ -175,6 +175,7 @@ struct supplicant_result {
 struct supplicant_task {
 	int ifindex;
 	char *ifname;
+	gboolean mac80211;
 	struct connman_device *device;
 	struct connman_network *network;
 	struct connman_network *pending_network;
@@ -533,7 +534,6 @@ static int remove_interface(struct supplicant_task *task)
 	return -EINPROGRESS;
 }
 
-#if 0
 static int set_ap_scan(struct supplicant_task *task)
 {
 	DBusMessage *message, *reply;
@@ -572,7 +572,6 @@ static int set_ap_scan(struct supplicant_task *task)
 
 	return 0;
 }
-#endif
 
 static int add_network(struct supplicant_task *task)
 {
@@ -715,18 +714,15 @@ static int select_network(struct supplicant_task *task)
 	return 0;
 }
 
-static int enable_network(struct supplicant_task *task)
+static int disconnect_network(struct supplicant_task *task)
 {
 	DBusMessage *message, *reply;
 	DBusError error;
 
 	DBG("task %p", task);
 
-	if (task->netpath == NULL)
-		return -EINVAL;
-
-	message = dbus_message_new_method_call(SUPPLICANT_NAME, task->netpath,
-					SUPPLICANT_INTF ".Network", "enable");
+	message = dbus_message_new_method_call(SUPPLICANT_NAME, task->path,
+				SUPPLICANT_INTF ".Interface", "disconnect");
 	if (message == NULL)
 		return -ENOMEM;
 
@@ -741,45 +737,7 @@ static int enable_network(struct supplicant_task *task)
 			connman_error("%s", error.message);
 			dbus_error_free(&error);
 		} else
-			connman_error("Failed to enable network");
-		dbus_message_unref(message);
-		return -EIO;
-	}
-
-	dbus_message_unref(message);
-
-	dbus_message_unref(reply);
-
-	return 0;
-}
-
-static int disable_network(struct supplicant_task *task)
-{
-	DBusMessage *message, *reply;
-	DBusError error;
-
-	DBG("task %p", task);
-
-	if (task->netpath == NULL)
-		return -EINVAL;
-
-	message = dbus_message_new_method_call(SUPPLICANT_NAME, task->netpath,
-					SUPPLICANT_INTF ".Network", "disable");
-	if (message == NULL)
-		return -ENOMEM;
-
-	dbus_message_set_auto_start(message, FALSE);
-
-	dbus_error_init(&error);
-
-	reply = dbus_connection_send_with_reply_and_block(connection,
-							message, -1, &error);
-	if (reply == NULL) {
-		if (dbus_error_is_set(&error) == TRUE) {
-			connman_error("%s", error.message);
-			dbus_error_free(&error);
-		} else
-			connman_error("Failed to disable network");
+			connman_error("Failed to disconnect network");
 		dbus_message_unref(message);
 		return -EIO;
 	}
@@ -823,13 +781,13 @@ static int set_network(struct supplicant_task *task,
 	connman_dbus_dict_append_variant(&dict, "scan_ssid",
 					 DBUS_TYPE_UINT32, &scan_ssid);
 
-	if (address)
+	if (network)
+		connman_dbus_dict_append_array(&dict, "ssid",
+						DBUS_TYPE_BYTE, &network, len);
+	else if (address)
 		connman_dbus_dict_append_variant(&dict, "bssid",
 						DBUS_TYPE_STRING, &address);
 
-	connman_dbus_dict_append_array(&dict, "ssid",
-					DBUS_TYPE_BYTE, &network, len);
-
 	if (g_ascii_strcasecmp(security, "wpa") == 0 ||
 				g_ascii_strcasecmp(security, "rsn") == 0) {
 		const char *key_mgmt = "WPA-PSK";
@@ -841,9 +799,12 @@ static int set_network(struct supplicant_task *task,
 						DBUS_TYPE_STRING, &passphrase);
 	} else if (g_ascii_strcasecmp(security, "wep") == 0) {
 		const char *key_mgmt = "NONE";
-		const char *auth_alg = "OPEN SHARED";
+		const char *auth_alg = "OPEN";
 		const char *key_index = "0";
 
+		if (task->mac80211 == TRUE)
+			auth_alg = "OPEN SHARED";
+
 		connman_dbus_dict_append_variant(&dict, "auth_alg",
 						DBUS_TYPE_STRING, &auth_alg);
 
@@ -937,6 +898,9 @@ static int initiate_scan(struct supplicant_task *task)
 
 	DBG("task %p", task);
 
+	if (task->path == NULL)
+		return -EINVAL;
+
 	if (task->scan_call != NULL)
 		return -EALREADY;
 
@@ -1547,14 +1511,15 @@ static int task_connect(struct supplicant_task *task)
 	unsigned int ssid_len;
 	int err;
 
+	connman_inet_ifup(task->ifindex);
+
 	address = connman_network_get_string(task->network, "Address");
 	security = connman_network_get_string(task->network, "WiFi.Security");
 	passphrase = connman_network_get_string(task->network, "WiFi.Passphrase");
 
 	ssid = connman_network_get_blob(task->network, "WiFi.SSID", &ssid_len);
 
-	DBG("address %s security %s passphrase %s",
-					address, security, passphrase);
+	DBG("address %s security %s", address, security);
 
 	if (security == NULL && passphrase == NULL)
 		return -EINVAL;
@@ -1562,14 +1527,15 @@ static int task_connect(struct supplicant_task *task)
 	if (g_str_equal(security, "none") == FALSE && passphrase == NULL)
 		return -EINVAL;
 
-	add_network(task);
+	remove_network(task);
 
-	select_network(task);
-	disable_network(task);
+	set_ap_scan(task);
+
+	add_network(task);
 
 	set_network(task, ssid, ssid_len, address, security, passphrase);
 
-	err = enable_network(task);
+	err = select_network(task);
 	if (err < 0)
 		return err;
 
@@ -1603,7 +1569,7 @@ static void state_change(struct supplicant_task *task, DBusMessage *msg)
 	const char *newstate, *oldstate;
 	unsigned char bssid[ETH_ALEN];
 	unsigned int bssid_len;
-	enum supplicant_state state;
+	enum supplicant_state state, prevstate;
 
 	dbus_error_init(&error);
 
@@ -1620,17 +1586,19 @@ static void state_change(struct supplicant_task *task, DBusMessage *msg)
 
 	DBG("state %s ==> %s", oldstate, newstate);
 
-	connman_info("%s %s", task->ifname, newstate);
+	connman_info("%s %s%s", task->ifname, newstate,
+				task->scanning == TRUE ? " (scanning)" : "");
 
 	state = string2state(newstate);
 	if (state == WPA_INVALID)
 		return;
 
 	if (task->scanning == TRUE && state != WPA_SCANNING) {
-		connman_device_set_scanning(task->device, FALSE);
+		connman_device_cleanup_scanning(task->device);
 		task->scanning = FALSE;
 	}
 
+	prevstate = task->state;
 	task->state = state;
 
 	if (task->network == NULL)
@@ -1638,8 +1606,19 @@ static void state_change(struct supplicant_task *task, DBusMessage *msg)
 
 	switch (task->state) {
 	case WPA_COMPLETED:
+		switch (prevstate) {
+		case WPA_ASSOCIATED:
+		case WPA_GROUP_HANDSHAKE:
+			break;
+		default:
+			goto badstate;
+		}
+
+		/* reset scan trigger and schedule background scan */
+		connman_device_schedule_scan(task->device);
+
 		if (get_bssid(task->device, bssid, &bssid_len) == 0)
-	                connman_network_set_address(task->network,
+			connman_network_set_address(task->network,
 							bssid, bssid_len);
 
 		/* carrier on */
@@ -1647,7 +1626,7 @@ static void state_change(struct supplicant_task *task, DBusMessage *msg)
 		break;
 
 	case WPA_DISCONNECTED:
-		disable_network(task);
+		remove_network(task);
 
 		/* carrier off */
 		connman_network_set_connected(task->network, FALSE);
@@ -1666,10 +1645,26 @@ static void state_change(struct supplicant_task *task, DBusMessage *msg)
 		break;
 
 	case WPA_ASSOCIATING:
-		connman_network_set_associating(task->network, TRUE);
+		switch (prevstate) {
+		case WPA_COMPLETED:
+			break;
+		case WPA_SCANNING:
+			connman_network_set_associating(task->network, TRUE);
+			break;
+		default:
+			goto badstate;
+		}
 		break;
 
 	case WPA_INACTIVE:
+		switch (prevstate) {
+		case WPA_SCANNING:
+		case WPA_DISCONNECTED:
+			break;
+		default:
+			goto badstate;
+		}
+
 		connman_network_set_connected(task->network, FALSE);
 
 		if (task->disconnecting == TRUE) {
@@ -1689,6 +1684,12 @@ static void state_change(struct supplicant_task *task, DBusMessage *msg)
 		connman_network_set_associating(task->network, FALSE);
 		break;
 	}
+
+	return;
+
+badstate:
+	connman_error("%s invalid state change %s -> %s", task->ifname,
+							oldstate, newstate);
 }
 
 static DBusHandlerResult supplicant_filter(DBusConnection *conn,
@@ -1744,6 +1745,10 @@ int supplicant_start(struct connman_device *device)
 		goto failed;
 	}
 
+	task->mac80211 = connman_inet_is_mac80211(task->ifindex);
+	if (task->mac80211 == FALSE)
+		connman_warn("Enabling quirks for unsupported driver");
+
 	task->range = g_try_malloc0(sizeof(struct iw_range));
 	if (task->range == NULL) {
 		err = -ENOMEM;
@@ -1802,10 +1807,10 @@ int supplicant_stop(struct connman_device *device)
 	if (task->scanning == TRUE)
 		connman_device_set_scanning(task->device, FALSE);
 
-	disable_network(task);
-
 	remove_network(task);
 
+	disconnect_network(task);
+
 	return remove_interface(task);
 }
 
@@ -1888,10 +1893,10 @@ int supplicant_disconnect(struct connman_network *network)
 	if (task->disconnecting == TRUE)
 		return -EALREADY;
 
-	disable_network(task);
-
 	remove_network(task);
 
+	disconnect_network(task);
+
 	task->disconnecting = TRUE;
 
 	return 0;
