@@ -45,6 +45,7 @@ struct connman_device {
 	connman_bool_t carrier;
 	connman_bool_t scanning;
 	connman_bool_t disconnected;
+	connman_bool_t reconnect;
 	connman_uint16_t scan_interval;
 	char *name;
 	char *node;
@@ -106,6 +107,19 @@ static void reset_scan_trigger(struct connman_device *device)
 	}
 }
 
+static void force_scan_trigger(struct connman_device *device)
+{
+	clear_scan_trigger(device);
+
+	device->scan_timeout = g_timeout_add_seconds(5,
+					device_scan_trigger, device);
+}
+
+void connman_device_schedule_scan(struct connman_device *device)
+{
+	reset_scan_trigger(device);
+}
+
 static const char *type2description(enum connman_device_type type)
 {
 	switch (type) {
@@ -123,6 +137,7 @@ static const char *type2description(enum connman_device_type type)
 	case CONNMAN_DEVICE_TYPE_GPS:
 		return "GPS";
 	case CONNMAN_DEVICE_TYPE_MBM:
+	case CONNMAN_DEVICE_TYPE_CELLULAR:
 	case CONNMAN_DEVICE_TYPE_HSO:
 	case CONNMAN_DEVICE_TYPE_NOZOMI:
 	case CONNMAN_DEVICE_TYPE_HUAWEI:
@@ -150,6 +165,7 @@ static const char *type2string(enum connman_device_type type)
 	case CONNMAN_DEVICE_TYPE_GPS:
 		return "gps";
 	case CONNMAN_DEVICE_TYPE_MBM:
+	case CONNMAN_DEVICE_TYPE_CELLULAR:
 	case CONNMAN_DEVICE_TYPE_HSO:
 	case CONNMAN_DEVICE_TYPE_HUAWEI:
 	case CONNMAN_DEVICE_TYPE_NOZOMI:
@@ -181,6 +197,7 @@ enum connman_service_type __connman_device_get_service_type(struct connman_devic
 	case CONNMAN_DEVICE_TYPE_BLUETOOTH:
 		return CONNMAN_SERVICE_TYPE_BLUETOOTH;
 	case CONNMAN_DEVICE_TYPE_MBM:
+	case CONNMAN_DEVICE_TYPE_CELLULAR:
 	case CONNMAN_DEVICE_TYPE_HSO:
 		return CONNMAN_SERVICE_TYPE_CELLULAR;
 	}
@@ -300,8 +317,6 @@ static int set_powered(struct connman_device *device, connman_bool_t powered)
 	if (device->powered_pending == powered)
 		return -EALREADY;
 
-	device->powered_pending = powered;
-
 	if (!driver)
 		return -EINVAL;
 
@@ -309,12 +324,18 @@ static int set_powered(struct connman_device *device, connman_bool_t powered)
 
 	if (powered == TRUE) {
 		if (driver->enable) {
+			device->powered_pending = powered;
+
 			err = driver->enable(device);
 			if (err == 0)
 				__connman_notifier_enable(type);
 		} else
 			err = -EINVAL;
 	} else {
+		device->powered_pending = powered;
+
+		device->reconnect = FALSE;
+
 		clear_scan_trigger(device);
 
 		g_hash_table_remove_all(device->networks);
@@ -853,6 +874,8 @@ static void device_destruct(struct connman_element *element)
 		device->timeout = 0;
 	}
 
+	clear_scan_trigger(device);
+
 	if (device->pending != NULL) {
 		dbus_message_unref(device->pending);
 		device->pending = NULL;
@@ -942,6 +965,7 @@ struct connman_device *connman_device_create(const char *node,
 		device->scan_interval = 0;
 		break;
 	case CONNMAN_DEVICE_TYPE_MBM:
+	case CONNMAN_DEVICE_TYPE_CELLULAR:
 	case CONNMAN_DEVICE_TYPE_HSO:
 	case CONNMAN_DEVICE_TYPE_NOZOMI:
 	case CONNMAN_DEVICE_TYPE_HUAWEI:
@@ -1230,6 +1254,8 @@ int __connman_device_set_blocked(struct connman_device *device,
 	if (device->offlinemode == TRUE)
 		return 0;
 
+	connman_info("%s {rfkill} blocked %d", device->interface, blocked);
+
 	if (blocked == FALSE)
 		powered = device->powered_persistent;
 	else
@@ -1275,6 +1301,8 @@ int __connman_device_scan(struct connman_device *device)
 	if (device->powered == FALSE)
 		return -ENOLINK;
 
+	reset_scan_trigger(device);
+
 	return device->driver->scan(device);
 }
 
@@ -1334,6 +1362,8 @@ int __connman_device_disable(struct connman_device *device)
 
 	device->powered_pending = FALSE;
 
+	device->reconnect = FALSE;
+
 	clear_scan_trigger(device);
 
 	g_hash_table_remove_all(device->networks);
@@ -1428,6 +1458,48 @@ void __connman_device_cleanup_networks(struct connman_device *device)
 					remove_unavailable_network, NULL);
 }
 
+static void scanning_changed(struct connman_device *device)
+{
+	DBusMessage *signal;
+	DBusMessageIter entry, value;
+	const char *key = "Scanning";
+
+	signal = dbus_message_new_signal(device->element.path,
+				CONNMAN_DEVICE_INTERFACE, "PropertyChanged");
+	if (signal == NULL)
+		return;
+
+	dbus_message_iter_init_append(signal, &entry);
+
+	dbus_message_iter_append_basic(&entry, DBUS_TYPE_STRING, &key);
+
+	dbus_message_iter_open_container(&entry, DBUS_TYPE_VARIANT,
+					DBUS_TYPE_BOOLEAN_AS_STRING, &value);
+	dbus_message_iter_append_basic(&value, DBUS_TYPE_BOOLEAN,
+							&device->scanning);
+	dbus_message_iter_close_container(&entry, &value);
+
+	g_dbus_send_message(connection, signal);
+}
+
+static void mark_network_available(gpointer key, gpointer value,
+						gpointer user_data)
+{
+	struct connman_network *network = value;
+
+	connman_network_set_available(network, TRUE);
+}
+
+void connman_device_cleanup_scanning(struct connman_device *device)
+{
+	device->scanning = FALSE;
+
+	scanning_changed(device);
+
+	g_hash_table_foreach(device->networks,
+				mark_network_available, NULL);
+}
+
 /**
  * connman_device_set_scanning:
  * @device: device structure
@@ -1438,10 +1510,6 @@ void __connman_device_cleanup_networks(struct connman_device *device)
 int connman_device_set_scanning(struct connman_device *device,
 						connman_bool_t scanning)
 {
-	DBusMessage *signal;
-	DBusMessageIter entry, value;
-	const char *key = "Scanning";
-
 	DBG("device %p scanning %d", device, scanning);
 
 	if (!device->driver || !device->driver->scan)
@@ -1452,21 +1520,7 @@ int connman_device_set_scanning(struct connman_device *device,
 
 	device->scanning = scanning;
 
-	signal = dbus_message_new_signal(device->element.path,
-				CONNMAN_DEVICE_INTERFACE, "PropertyChanged");
-	if (signal == NULL)
-		return 0;
-
-	dbus_message_iter_init_append(signal, &entry);
-
-	dbus_message_iter_append_basic(&entry, DBUS_TYPE_STRING, &key);
-
-	dbus_message_iter_open_container(&entry, DBUS_TYPE_VARIANT,
-					DBUS_TYPE_BOOLEAN_AS_STRING, &value);
-	dbus_message_iter_append_basic(&value, DBUS_TYPE_BOOLEAN, &scanning);
-	dbus_message_iter_close_container(&entry, &value);
-
-	g_dbus_send_message(connection, signal);
+	scanning_changed(device);
 
 	if (scanning == TRUE) {
 		reset_scan_trigger(device);
@@ -1516,6 +1570,9 @@ int connman_device_set_disconnected(struct connman_device *device,
 
 	device->disconnected = disconnected;
 
+	if (disconnected == TRUE)
+		force_scan_trigger(device);
+
 	return 0;
 }
 
@@ -1723,6 +1780,11 @@ int connman_device_remove_network(struct connman_device *device,
 	return 0;
 }
 
+void connman_device_remove_all_networks(struct connman_device *device)
+{
+	g_hash_table_remove_all(device->networks);
+}
+
 void __connman_device_set_network(struct connman_device *device,
 					struct connman_network *network)
 {
@@ -1749,6 +1811,18 @@ void __connman_device_set_network(struct connman_device *device,
 	}
 }
 
+void __connman_device_set_reconnect(struct connman_device *device,
+						connman_bool_t reconnect)
+{
+	device->reconnect = reconnect;
+}
+
+connman_bool_t  __connman_device_get_reconnect(
+				struct connman_device *device)
+{
+	return device->reconnect;
+}
+
 /**
  * connman_device_register:
  * @device: device structure
