@@ -37,6 +37,7 @@ static DBusConnection *connection;
 static GNode *element_root = NULL;
 static GSList *driver_list = NULL;
 static gchar *device_filter = NULL;
+static gchar *nodevice_filter = NULL;
 
 static gboolean started = FALSE;
 
@@ -76,27 +77,6 @@ static const char *type2string(enum connman_element_type type)
 	return NULL;
 }
 
-static void emit_element_signal(DBusConnection *conn, const char *member,
-					struct connman_element *element)
-{
-	DBusMessage *signal;
-
-	if (__connman_debug_enabled() == FALSE)
-		return;
-
-	DBG("conn %p member %s", conn, member);
-
-	if (element == NULL)
-		return;
-
-	signal = dbus_message_new_signal(element->path,
-					CONNMAN_DEBUG_INTERFACE, member);
-	if (signal == NULL)
-		return;
-
-	g_dbus_send_message(conn, signal);
-}
-
 struct foreach_data {
 	enum connman_element_type type;
 	element_cb_t callback;
@@ -284,6 +264,7 @@ struct connman_service *__connman_element_get_service(struct connman_element *el
 	case CONNMAN_DEVICE_TYPE_WIMAX:
 	case CONNMAN_DEVICE_TYPE_BLUETOOTH:
 	case CONNMAN_DEVICE_TYPE_MBM:
+	case CONNMAN_DEVICE_TYPE_CELLULAR:
 	case CONNMAN_DEVICE_TYPE_HSO:
 		network = __connman_element_get_network(element);
 		if (network == NULL)
@@ -1363,14 +1344,34 @@ static void register_element(gpointer data, gpointer user_data)
 			emit_state_change(connection, "online");
 	}
 
-	emit_element_signal(connection, "ElementAdded", element);
-
 	if (started == FALSE)
 		return;
 
 	probe_element(element);
 }
 
+gboolean __connman_element_device_isfiltered(const char *devname)
+{
+	if (device_filter == NULL)
+		goto nodevice;
+
+	if (g_pattern_match_simple(device_filter, devname) == FALSE) {
+		DBG("ignoring device %s (match)", devname);
+		return TRUE;
+	}
+
+nodevice:
+	if (nodevice_filter == NULL)
+		return FALSE;
+
+	if (g_pattern_match_simple(nodevice_filter, devname) == TRUE) {
+		DBG("ignoring device %s (no match)", devname);
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 /**
  * connman_element_register:
  * @element: the element to register
@@ -1389,15 +1390,13 @@ int connman_element_register(struct connman_element *element,
 	if (element->devname == NULL)
 		element->devname = g_strdup(element->name);
 
-	if (device_filter && element->type == CONNMAN_ELEMENT_TYPE_DEVICE) {
-		if (g_pattern_match_simple(device_filter,
-						element->devname) == FALSE) {
-			DBG("ignoring %s [%s] device", element->name,
-							element->devname);
-			return -EPERM;
-		}
-	}
+        if (element->type != CONNMAN_ELEMENT_TYPE_DEVICE)
+		goto setup;
 
+	if (__connman_element_device_isfiltered(element->devname) == TRUE)
+		return -EPERM;
+
+setup:
 	if (connman_element_ref(element) == NULL)
 		return -EINVAL;
 
@@ -1454,8 +1453,6 @@ static gboolean remove_element(GNode *node, gpointer user_data)
 			emit_state_change(connection, "offline");
 	}
 
-	emit_element_signal(connection, "ElementRemoved", element);
-
 	connman_element_unref(element);
 
 	return FALSE;
@@ -1496,8 +1493,6 @@ static gboolean update_element(GNode *node, gpointer user_data)
 	if (element->driver && element->driver->update)
 		element->driver->update(element);
 
-	emit_element_signal(connection, "ElementUpdated", element);
-
 	return FALSE;
 }
 
@@ -1580,6 +1575,7 @@ int __connman_element_init(const char *device, const char *nodevice)
 		return -EIO;
 
 	device_filter = g_strdup(device);
+	nodevice_filter = g_strdup(nodevice);
 
 	element = connman_element_create("root");
 
@@ -1692,6 +1688,7 @@ void __connman_element_cleanup(void)
 	g_node_destroy(element_root);
 	element_root = NULL;
 
+	g_free(nodevice_filter);
 	g_free(device_filter);
 
 	if (connection == NULL)
