@@ -39,6 +39,7 @@ struct connman_network {
 	connman_bool_t secondary;
 	connman_bool_t available;
 	connman_bool_t connected;
+	connman_bool_t roaming;
 	connman_bool_t hidden;
 	connman_uint8_t strength;
 	connman_uint16_t frequency;
@@ -81,6 +82,7 @@ static const char *type2string(enum connman_network_type type)
 	case CONNMAN_NETWORK_TYPE_BLUETOOTH_PAN:
 	case CONNMAN_NETWORK_TYPE_BLUETOOTH_DUN:
 		return "bluetooth";
+	case CONNMAN_NETWORK_TYPE_CELLULAR:
 	case CONNMAN_NETWORK_TYPE_MBM:
 	case CONNMAN_NETWORK_TYPE_HSO:
 		return "cellular";
@@ -554,6 +556,7 @@ void connman_network_set_group(struct connman_network *network,
 		return;
 	case CONNMAN_NETWORK_TYPE_BLUETOOTH_PAN:
 	case CONNMAN_NETWORK_TYPE_BLUETOOTH_DUN:
+	case CONNMAN_NETWORK_TYPE_CELLULAR:
 	case CONNMAN_NETWORK_TYPE_MBM:
 	case CONNMAN_NETWORK_TYPE_HSO:
 	case CONNMAN_NETWORK_TYPE_WIFI:
@@ -608,6 +611,7 @@ connman_bool_t __connman_network_get_weakness(struct connman_network *network)
 	case CONNMAN_NETWORK_TYPE_VENDOR:
 	case CONNMAN_NETWORK_TYPE_BLUETOOTH_PAN:
 	case CONNMAN_NETWORK_TYPE_BLUETOOTH_DUN:
+	case CONNMAN_NETWORK_TYPE_CELLULAR:
 	case CONNMAN_NETWORK_TYPE_MBM:
 	case CONNMAN_NETWORK_TYPE_HSO:
 	case CONNMAN_NETWORK_TYPE_WIMAX:
@@ -683,6 +687,35 @@ int connman_network_set_associating(struct connman_network *network,
 	return 0;
 }
 
+static void set_associate_error(struct connman_network *network)
+{
+	struct connman_service *service;
+
+	if (network->associating == FALSE)
+		return ;
+
+	network->associating = FALSE;
+
+	service = __connman_service_lookup_from_network(network);
+
+	__connman_service_indicate_state(service,
+					CONNMAN_SERVICE_STATE_FAILURE);
+}
+
+void connman_network_set_error(struct connman_network *network,
+					enum connman_network_error error)
+{
+	DBG("nework %p, error %d", network, error);
+
+	switch (error) {
+	case CONNMAN_NETWORK_ERROR_UNKNOWN:
+		return;
+	case CONNMAN_NETWORK_ERROR_ASSOCIATE_FAIL:
+		set_associate_error(network);
+		break;
+	}
+}
+
 static gboolean set_connected(gpointer user_data)
 {
 	struct connman_network *network = user_data;
@@ -963,6 +996,24 @@ int connman_network_set_strength(struct connman_network *network,
 						"Strength", strength);
 }
 
+/**
+ * connman_network_set_roaming:
+ * @network: network structure
+ * @roaming: roaming state
+ *
+ * Set roaming state for network
+ */
+int connman_network_set_roaming(struct connman_network *network,
+						connman_bool_t roaming)
+{
+	DBG("network %p roaming %d", network, roaming);
+
+	network->roaming = roaming;
+
+	return connman_element_set_bool(&network->element,
+						"Roaming", roaming);
+}
+
 /**
  * connman_network_set_string:
  * @network: network structure
@@ -1027,6 +1078,43 @@ const char *connman_network_get_string(struct connman_network *network,
 	return connman_element_get_string(&network->element, key);
 }
 
+/**
+ * connman_network_set_bool:
+ * @network: network structure
+ * @key: unique identifier
+ * @value: boolean value
+ *
+ * Set boolean value for specific key
+ */
+int connman_network_set_bool(struct connman_network *network,
+					const char *key, connman_bool_t value)
+{
+	DBG("network %p key %s value %d", network, key, value);
+
+	if (g_strcmp0(key, "Roaming") == 0)
+		return connman_network_set_roaming(network, value);
+
+	return connman_element_set_bool(&network->element, key, value);
+}
+
+/**
+ * connman_network_get_bool:
+ * @network: network structure
+ * @key: unique identifier
+ *
+ * Get boolean value for specific key
+ */
+connman_bool_t connman_network_get_bool(struct connman_network *network,
+							const char *key)
+{
+	DBG("network %p key %s", network, key);
+
+	if (g_str_equal(key, "Roaming") == TRUE)
+		return network->roaming;
+
+	return connman_element_get_bool(&network->element, key);
+}
+
 /**
  * connman_network_set_uint8:
  * @network: network structure
@@ -1250,6 +1338,7 @@ static int network_probe(struct connman_element *element)
 		break;
 	case CONNMAN_NETWORK_TYPE_BLUETOOTH_PAN:
 	case CONNMAN_NETWORK_TYPE_BLUETOOTH_DUN:
+	case CONNMAN_NETWORK_TYPE_CELLULAR:
 	case CONNMAN_NETWORK_TYPE_MBM:
 	case CONNMAN_NETWORK_TYPE_HSO:
 	case CONNMAN_NETWORK_TYPE_WIFI:
@@ -1280,6 +1369,7 @@ static void network_remove(struct connman_element *element)
 		break;
 	case CONNMAN_NETWORK_TYPE_BLUETOOTH_PAN:
 	case CONNMAN_NETWORK_TYPE_BLUETOOTH_DUN:
+	case CONNMAN_NETWORK_TYPE_CELLULAR:
 	case CONNMAN_NETWORK_TYPE_MBM:
 	case CONNMAN_NETWORK_TYPE_HSO:
 	case CONNMAN_NETWORK_TYPE_WIFI:
