@@ -110,6 +110,10 @@ static struct connman_rtnl detect_rtnl = {
 	.dellink	= detect_dellink,
 };
 
+void __connman_udev_enable_rfkill_processing(void)
+{
+}
+
 char *__connman_udev_get_devtype(const char *ifname)
 {
 	return NULL;
@@ -120,19 +124,21 @@ char *__connman_udev_get_mbm_devnode(const char *ifname)
 	return NULL;
 }
 
+void __connman_udev_rfkill(const char *sysname, connman_bool_t blocked)
+{
+	DBG("sysname %s blocked %d", sysname, blocked);
+}
+
 int __connman_udev_init(void)
 {
-	int err;
-
 	DBG("");
 
-	err = connman_rtnl_register(&detect_rtnl);
-	if (err < 0)
-		return err;
+	return connman_rtnl_register(&detect_rtnl);
+}
 
-	connman_rtnl_send_getlink();
-
-	return 0;
+void __connman_udev_start(void)
+{
+	DBG("");
 }
 
 void __connman_udev_cleanup(void)
